
//-------------- Call Object
// Manages call state + UI

Call.MAXCALLS = 4;
Call.selected = -1;
Call.calls    = 0; /* Active calls */

// Access localized strings: stringbundle must be defined in document
var locales;

function Call( n, loc ) { // Constructor
	locales = document.getElementById( loc );
	// Generate UI
/* This is (hopefully !) the resulting XUL
	<groupbox style="-moz-border-radius: 20px; background: #aaaaaa">
		<vbox>
			<description id="cap_call_0" value="Ligne 1" class="LCD_off" />
			<description id="lduration0" value=" 0:00" class="LCD_off" />
			<hbox>
				<button id="bselect0" label="S" tooltiptext="Selectionner"
					oncommand="calls[0].select()" class="digits" />
				<button id="bcall0" label="R" tooltiptext="Racrocher"
					oncommand="calls[0].hangup()" class="digits" />
				<button id="bcall0" label="P" tooltiptext="Muet"
					oncommand="calls[0].mute()" class="digits" />
			</hbox>
		</vbox>
	</groupbox>
*/
	var gb = document.createElement( 'groupbox' );
	gb.setAttribute( 'style', '-moz-border-radius: 20px; background: #aaaaaa' );
	gb.setAttribute( 'id', 'call_group_' + n );
	gb.setAttribute( 'collapsed', false ); 
	gb.setAttribute( 'tooltiptext', locales.getString('select.tt') );
	gb.setAttribute( 'onclick', 'calls[' + n + '].select()' );
	this.ui = gb;
	var vb = document.createElement( 'vbox' );
	gb.appendChild( vb );
	var d = document.createElement( 'description' );
	d.setAttribute( 'id', 'cap_call_' + n );
	d.setAttribute( 'value', 'Ligne ' + parseInt(n+1) );
	d.setAttribute( 'class', 'LCD_off' );
	vb.appendChild( d );
	d = document.createElement( 'description' );
	d.setAttribute( 'id', 'lduration' + n );
	d.setAttribute( 'value', ' 0:00' );
	d.setAttribute( 'class', 'LCD_off' );
	vb.appendChild( d );
	this.lduration = d;

	var hb = document.createElement( 'hbox' );
	vb.appendChild( hb );
	var b = document.createElement( 'button' );
	b.setAttribute( 'label', locales.getString('text.label') );
	b.setAttribute( 'tooltiptext', locales.getString('text.tt') );
	b.setAttribute( 'oncommand', 'calls[' + n + '].text()' );
	b.setAttribute( 'class', 'digits' );
	hb.appendChild( b ); 
	b = document.createElement( 'button' );
	b.setAttribute( 'label', locales.getString('hangup.label') );
	b.setAttribute( 'tooltiptext', locales.getString('hangup.tt') );
	b.setAttribute( 'oncommand', 'calls[' + n + '].hangup()' );
	b.setAttribute( 'class', 'digits' );
	hb.appendChild( b );
	b = document.createElement( 'button' );
	b.setAttribute( 'type', 'checkbox' );
	b.setAttribute( 'id', 'mute_call_' + n );
	b.setAttribute( 'label', locales.getString('mute.label') );
	b.setAttribute( 'tooltiptext', locales.getString('mute.tt') );
	b.setAttribute( 'oncommand', 'calls[' + n + '].mute()' );
	b.setAttribute( 'class', 'digits' );
	hb.appendChild( b );

	// Properties
	this.callNo     = n;
	this.active     = false;
	this.begin      = new Date();
	this.accept_dlg = false;
	this.timer	    = 0;

	// Methods
	this.activated     = activated;
	this.select        = select;
	this.text          = text;
	this.selected      = selected;
	this.hangup        = hangup;
	this.mute          = mute;
	this.disconnected  = disconnected;
	this.ringing       = ringing;
	this.dur_timer     = dur_timer;
	this.accept_dlg_cb = accept_dlg_cb;
	this.ringout       = ringout;
	this.show          = show;
	this.hide          = hide;

}

function select() { // Called when user click on Select button
	proxiax_send( 'select,' + this.callNo );
	Call.selected = this.callNo;
}

function selected() { // Called on IAXclient select event
	if( ! this.active ) return;
	Call.selected = this.callNo;
	for( i=0; i<Call.MAXCALLS; i++ ) {
		if( i == this.callNo ) { continue; }
		if( document.getElementById('cap_call_' + i ) ) {
			var x = document.getElementById('cap_call_' + i );
			switch( x.getAttribute('class') ) {
				case 'LCD_mute':
					proxiax_send( 'unmute,' + i );
					var b = document.getElementById('mute_call_'+ i );
					b.setAttribute( 'label', locales.getString('mute.label') );
					b.setAttribute( 'tooltiptext', locales.getString('mute.tt') );
					b.setAttribute( 'checked', 'false' );
					// Fall through, no break
				case 'LCD_on':
					document.getElementById('cap_call_' + i ).setAttribute('class', 'LCD_hold');
					document.getElementById('lduration' + i ).setAttribute('class', 'LCD_hold');
					break;
			}
		}
	}
	document.getElementById( 'call_group_' + this.callNo ).setAttribute( 'style', '-moz-border-radius: 20px; background: #cccccc' );
	document.getElementById('cap_call_' + this.callNo).setAttribute('class', 'LCD_on');
	document.getElementById('lduration' + this.callNo).setAttribute('class', 'LCD_on');
}

function hangup() { // Called when user click on Hangup button
	if( prefs.callconf=='true' && 
			!confirm( locales.getString('confirm_hangup' ) ) )
		return;
	proxiax_send( 'hangup,' + this.callNo );
}

function text() { // Called when user click on Text button
	if( s = prompt( locales.getString('text.prompt') ) ) {
		proxiax_send( 'text,' + this.callNo + ',' + s );
	}
}

function mute() { // Called when user click on Mute button
	if( ! this.active ) return;
	n = this.callNo;
	var b = document.getElementById('mute_call_'+ n );
	if( b.getAttribute('checked') == 'true' ) {
		proxiax_send( 'unmute,' + this.callNo );
		b.setAttribute( 'label', locales.getString('mute.label') );
		b.setAttribute( 'tooltiptext', locales.getString('mute.tt') );
		document.getElementById('cap_call_' + n ).setAttribute('class', 'LCD_on');
		document.getElementById('lduration' + n ).setAttribute('class', 'LCD_on');
	} else {
		proxiax_send( 'mute,' + this.callNo );
		b.setAttribute( 'label', locales.getString('unmute.label') );
		b.setAttribute( 'tooltiptext', locales.getString('unmute.tt') );
		document.getElementById('cap_call_' + n ).setAttribute('class', 'LCD_mute');
		document.getElementById('lduration' + n ).setAttribute('class', 'LCD_mute');
	}
}

function disconnected() { // Called on IAXclient not_active event
	if( this.accept_dlg ) this.accept_dlg.close();
	if( ! this.active ) return;
	Call.selected = -1;
	Call.calls--;
	this.active = false;
	var d = document.getElementById('cap_call_'+ this.callNo);
	if( d ) {
		d.setAttribute('class', 'LCD_off');
		d = document.getElementById('lduration' + this.callNo);
		d.setAttribute('class', 'LCD_off');
		document.getElementById( 'call_group_' + this.callNo ).setAttribute( 'style', '-moz-border-radius: 20px; background: #aaaaaa' );
	}
	if( prefs.autohide=='true' ) {
		// Need to use closure for timeout on objects
		var call = this;
		setTimeout( function() { call.hide(); }, 5000);
	}
}

function ringing( remote ) {
	if( Call.calls == Call.MAXCALLS ) {
		proxiax_send( 'reject,' + this.callNo );
		return;
	}
	// Add incoming call to history
	var r = remote.split(':');
	append_call_list( '<', r[0] );
	param = new Object(); // params for dialog
	param.remote = remote;
	param.line   = this.callNo;
	param.call   = this;
	// Dialog calls accept_dlg_cb when closed
	this.accept_dlg = window.openDialog('chrome://mozphone/content/accept_dlg.xul', 'accept_dlg', 'chrome,resizable,scrollbars,dialog=yes,close', param );
}

function accept_dlg_cb( ret ) { // Called when accept_dlg is closed
	if( ret == 1 ) 
		proxiax_send( 'answer,' + this.callNo );
	else if( ret == 0 )
		proxiax_send( 'reject,' + this.callNo );
}

function activated( remote ) {
	this.show();
	if( ! this.active ) {
		this.begin = new Date();
		var d = document.getElementById('cap_call_' + this.callNo);
		d.value = remote;
		d.setAttribute('class', 'LCD_on');
		d = document.getElementById('lduration' + this.callNo);
		d.setAttribute('class', 'LCD_on');
		this.active = true;
		Call.calls++;
	}
	this.selected();
}

// This is called from timer in mozphone.js (proxyax_read), for efficiency reasons,
// (just one global timer instead of MAXCALLS timers).
function dur_timer() {
	if( !this.active ) return;
	now = new Date();
	var dur = (now-this.begin) / 1000;
	var s = Math.floor( dur % 60 );
	var m = Math.floor( dur / 60 );
	if( 10 > s ) s = '0' + s;
	this.lduration.value = m.toString() + ':' + s;
}

function ringout() {
	this.show();
}

function show() {
	var el = document.getElementById( 'call_group_' + this.callNo );
	if( ! el ) {
		// Insert the call UI main_vbox is the parent element
		document.getElementById('main_vbox')
			.insertBefore( this.ui, document.getElementById('dialgroup') );
		self.resizeBy( 0, 10 + this.ui.boxObject.height );
	} else {
		if( el.getAttribute('collapsed') == 'true' ) {
			collapse( 'call_group_' + this.callNo);
		}
	}
}

function hide() {
	if( ! this.active ) 
		collapse( 'call_group_' + this.callNo );
}

//-------------------------------------------------------------------
